# Generated by Django 2.1.1 on 2018-09-27 17:04

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, help_text='A name to identify this template', max_length=100, unique=True)),
                ('content', models.TextField(help_text='The full Django/Jinja template to be used.')),
                ('description', models.CharField(blank=True, max_length=200, null=True)),
            ],
        ),
    ]
